/******************************************************************* 
* Copyright (c) 2010 by Hesine Technologies, Inc. 
* All rights reserved. 
* 
* This file is proprietary and confidential to Hesine Technologies. 
* No part of this file may be reproduced, stored, transmitted, 
* disclosed or used in any form or by any means other than as 
* expressly provided by the written permission from Jianhui Tao 
* 
* ****************************************************************/

#ifndef _hpns_network_h_
#define _hpns_network_h_

typedef struct SNmsSocket_d SNmsSocket ;

#ifdef __ANDROIDOS__
struct SNmsSocket_d {
    int                sockFd ;  /* the handler of socket fd */
    unsigned int  incId ;     /* the auto increase id for the socket to check equal */
} ;
#endif 

/* reset socket to the invalid status */
void hpnsSocketReset(SNmsSocket* nmsSocket) ;

/* check if the socket is valid */
int hpnsSocketIsValid(const SNmsSocket* nmsSocket) ;

/* check if the two socket is equal */
int hpnsSocketIsEqual(const SNmsSocket* socket1, const SNmsSocket* socket2) ;


/*=====================================================================================
* function: hpnsGetTtl()
* note: 
*	1. it is used to get time to live(TTL)
*	2. returns TTL, the default value of TTL is 128.
*	
=====================================================================================*/
int    hpnsGetTtl(void);

/*=====================================================================================
* function: hpnsOpenUdpSocket( OUT char * asyncFlag, OUT SNmsSocket* nmsSocket)
* note:
*	1. it is used to open a UDP socket, returns 0 (succeed)  or -1(failed)
*	2. platform should calculate IAP itself
*	3. if this function is asynchronous, asyncFlag is set to 1, or it is 0.
*	4. if the socket is opened successfully, the sockFd will be set.
*	
=====================================================================================*/
int    hpnsOpenUdpSocket(char *asyncFlag, SNmsSocket* nmsSocket);

/*=====================================================================================
* function: hpnsCloseUdpSocket( IN SNmsSocket* nmsSocket)
* note:
*	1. it is used to close UDP socket
*	
=====================================================================================*/
void   hpnsCloseUdpSocket(SNmsSocket* nmsSocket);

/*=====================================================================================
* function: hpnsSendUdpData(IN SNmsSocket* nmsSocket, IN unsigned int ip, IN unsigned short port, IN unsigned char *msg, IN int msgLen); 
* note:
*	1. it s used to send UDP data.
*	2. In the parameters, ip and port must be network order
*	3. It returns bytes that have been sent normally, and if failed to send data, returns negative error number; 
*	if the socket is blocking, returns 0.
*	
=====================================================================================*/
int    hpnsSendUdpData(SNmsSocket* nmsSocket, unsigned int ip, unsigned short port, unsigned char *msg, int msgLen); 


/*=====================================================================================
* function: hpnsRecvUdpData(IN SNmsSocket* nmsSocket, OUT unsigned char *buffer,IN  int bufferLen, OUT unsigned int *ip, OUT unsigned short *port);
* note:
*	1. it is used to read UDP data
*	2.  In the parameters, bufferLen is bytes to read, ip and port are the server's address
*	3.  It returns bytes that have been read, if failed to read data, returns a negative error number; 
*	and if socket is blocking, returns 0.
*	
=====================================================================================*/
int    hpnsRecvUdpData(SNmsSocket* nmsSocket, unsigned char *buffer, int bufferLen, unsigned int *ip, unsigned short *port);

/*=====================================================================================
* function: hpnsGetServerIpViaDNS(IN char domain[])
* note:
*	1. it is used to get IP address by domain name server
*	2. it returns IP address which is in network order
*	3. This function may be an asynchronous operation. If it is asynchronous, returns -1,
*	and need to call the API that engine providesint hpnsProcessDnsResult(UINT32 ip).
*	
=====================================================================================*/
UINT32 hpnsGetServerIpViaDNS(char domain[]); 

/*=====================================================================================
* function: hpnsOpenTcpSocket(OUT SNmsSocket* nmsSocket)
* note:
*	1. it is used to open a TCP socket, returns 0 (succeed)  or -1(failed)
*	2. platform should calculate IAP itself
*	
=====================================================================================*/
int hpnsOpenTcpSocket(SNmsSocket* nmsSocket);

/*=====================================================================================
* function: hpnsCloseTcpSocket( IN SNmsSocket* nmsSocket)
* note:
*	1. it is used to close TCP socket
*	
=====================================================================================*/
void  hpnsCloseTcpSocket(SNmsSocket* nmsSocket);

/*=====================================================================================
* function: hpnsConnectTcpSocket(IN SNmsSocket* nmsSocket, IN unsigned int ip,IN UINT16 port)
* note:
*	1. it is used to connect TCP
*     2. it returns 1 when connecting succssfully;it returns 0 if socket is SOC_WOULDBLOCK, and returns negative error number if fialed
*	
=====================================================================================*/
int hpnsConnectTcpSocket(SNmsSocket* nmsSocket, unsigned int ip, UINT16 port);

/*=====================================================================================
* function: hpnsRecvTcpData(IN SNmsSocket* nmsSocket, IN unsigned char *msg, IN int msgLen)
* note:
*	1. it is used read TCP data
*     2.  In the parameters, msgLen is bytes to read
*	3.  It returns bytes that have been read, if failed to read data, returns a negative error number; 
*	and if socket is blocking, returns 0.
*	
=====================================================================================*/
int hpnsRecvTcpData(SNmsSocket* nmsSocket, unsigned char *msg, int msgLen);

/*=====================================================================================
* function: hpnsSendTcpData(IN SNmsSocket* nmsSocket, IN unsigned char *msg, IN int msgLen);
* note:
*	1. it is used to write TCP socket
*     2.In the parameters, msgLen is bytes to write
*	3.It returns bytes that have been sent normally, and if failed to send data, returns negative error number; 
*	if the socket is blocking, returns 0.
*	
=====================================================================================*/
int hpnsSendTcpData(SNmsSocket* nmsSocket, unsigned char *msg, int msgLen);


#endif
